"""
This example demonstrates how to skip the checks on whether the s3 folder. This
is not recommended and should be used sparingly. Please ask a Data Admin to
remove the data from S3 instead.
"""

from datetime import datetime

import requests
from aind_data_schema_models.modalities import Modality
from aind_data_schema_models.platforms import Platform

from aind_data_transfer_service.models.core import (
    SubmitJobRequestV2,
    Task,
    UploadJobConfigsV2,
)

# The job_type contains the default settings for compression and Code Ocean
# pipelines.
job_type = "ecephys"

acq_datetime = datetime(2023, 4, 3, 18, 17, 7)

ecephys_task = Task(
    job_settings={
        "input_source": (
            "/allen/aind/scratch/svc_aind_upload/test_data_sets/"
            "ecephys/655019_2023-04-03_18-17-07"
        )
    }
)

modality_transformation_settings = {"ecephys": ecephys_task}

gather_preliminary_metadata = Task(
    job_settings={
        "metadata_dir": (
            "/allen/aind/scratch/svc_aind_upload/test_data_sets/"
            "ecephys/655019_2023-04-03_18-17-07"
        )
    }
)

upload_job_configs_v2 = UploadJobConfigsV2(
    job_type=job_type,
    project_name="Ephys Platform",
    platform=Platform.ECEPHYS,
    modalities=[Modality.ECEPHYS],
    subject_id="655019",
    acq_datetime=acq_datetime,
    tasks={
        "modality_transformation_settings": modality_transformation_settings,
        "check_s3_folder_exists": {"skip_task": True},
        "final_check_s3_folder_exist": {"skip_task": True},
        "gather_preliminary_metadata": gather_preliminary_metadata,
    },
)

submit_request_v2 = SubmitJobRequestV2(
    upload_jobs=[upload_job_configs_v2],
)

post_request_content = submit_request_v2.model_dump(
    mode="json", exclude_none=True
)

# Please use the production endpoint for submitting jobs and the dev endpoint
# for running tests.
# endpoint = "http://aind-data-transfer-service"
endpoint = "http://aind-data-transfer-service-dev"  # For testing

submit_job_response = requests.post(
    url=f"{endpoint}/api/v2/submit_jobs",
    json=post_request_content,
)
print(submit_job_response.status_code)
print(submit_job_response.json())



"""
This example demonstrates how to perform behavior video compression when a
predefined job_type is not available. However, it might be better to request a
job_type to be stored.
"""

from datetime import datetime

import requests
from aind_data_schema_models.modalities import Modality
from aind_data_schema_models.platforms import Platform

from aind_data_transfer_service.models.core import (
    SubmitJobRequestV2,
    Task,
    UploadJobConfigsV2,
)

# As default, behavior videos are not compressed during upload.
# This example demonstrates how to override the default behavior.
job_type = "default"

acq_datetime = datetime(2024, 5, 9, 12, 31, 22)

behavior_task = Task(
    job_settings={
        "input_source": (
            "/allen/aind/scratch/svc_aind_upload/test_data_sets/"
            "behavior_711256_2024-05-09_12-31-22/behavior"
        )
    }
)

# Using a predefined job_type would simplify this model greatly.
# Alternatively, this model can be validated against the JobSettings
# pydantic class in the aind-behavior-video-transformation package.
behavior_video_job_settings = {
    "input_source": (
        "/allen/aind/scratch/svc_aind_upload/test_data_sets/"
        "behavior_711256_2024-05-09_12-31-22/behavior-videos"
    ),
    "output_directory": "%OUTPUT_LOCATION",  # This will be replaced by svc
    "compression_requested": {"compression_enum": "gamma fix colorspace"},
}
behavior_videos_task = {
    "skip_task": False,
    "image": "ghcr.io/allenneuraldynamics/aind-behavior-video-transformation",
    "image_version": "0.1.7",
    "image_resources": {
        "memory_per_cpu": {"set": True, "number": 250},
        "minimum_cpus_per_node": 32,
        "partition": "aind",
        "qos": "dev",
        "standard_error": (
            "/allen/aind/scratch/svc_aind_airflow/dev/logs/%x_%j_error.out"
        ),
        "tasks": 1,
        "time_limit": {"set": True, "number": 720},
        "standard_output": (
            "/allen/aind/scratch/svc_aind_airflow/dev/logs/%x_%j.out"
        ),
        "environment": [
            "PATH=/bin:/usr/bin/:/usr/local/bin/",
            "LD_LIBRARY_PATH=/lib/:/lib64/:/usr/local/lib",
        ],
        "current_working_directory": ".",
    },
    "job_settings": behavior_video_job_settings,
    "command_script": (
        "#!/bin/bash \nsingularity exec --cleanenv "
        "docker://%IMAGE:%IMAGE_VERSION python -m "
        "aind_behavior_video_transformation.etl "
        "--job-settings ' %JOB_SETTINGS '"
    ),
}

modality_transformation_settings = {
    "behavior": behavior_task,
    "behavior-videos": behavior_videos_task,
}

gather_preliminary_metadata = Task(
    job_settings={
        "metadata_dir": (
            "/allen/aind/scratch/svc_aind_upload/test_data_sets/"
            "behavior_711256_2024-05-09_12-31-22/metadata-dir"
        )
    }
)


upload_job_configs_v2 = UploadJobConfigsV2(
    job_type=job_type,
    project_name="Behavior Platform",
    platform=Platform.BEHAVIOR,
    modalities=[Modality.BEHAVIOR, Modality.BEHAVIOR_VIDEOS],
    subject_id="711256",
    acq_datetime=acq_datetime,
    tasks={
        "modality_transformation_settings": modality_transformation_settings,
        "gather_preliminary_metadata": gather_preliminary_metadata,
    },
)

submit_request_v2 = SubmitJobRequestV2(
    upload_jobs=[upload_job_configs_v2],
)

post_request_content = submit_request_v2.model_dump(
    mode="json", exclude_none=True
)

# Please use the production endpoint for submitting jobs and the dev endpoint
# for running tests.
# endpoint = "http://aind-data-transfer-service"
endpoint = "http://aind-data-transfer-service-dev"  # For testing

submit_job_response = requests.post(
    url=f"{endpoint}/api/v2/submit_jobs",
    json=post_request_content,
)
print(submit_job_response.status_code)
print(submit_job_response.json())



"""
This example demonstrates how to pass in custom job_settings into the metadata
mapper package. It assumes a user has pip installed the aind-metadata-mapper
package. If there are dependency conflicts it may also be possible to use raw
dictionaries.
"""

from datetime import datetime

import requests
from aind_data_schema_models.modalities import Modality
from aind_data_schema_models.platforms import Platform
from aind_metadata_mapper.bergamo.models import (
    JobSettings as BergamoSessionSettings,
)
from aind_metadata_mapper.models import (
    JobSettings as GatherMetadataJobSettings,
)
from aind_metadata_mapper.models import SessionSettings

from aind_data_transfer_service.models.core import (
    SubmitJobRequestV2,
    Task,
    UploadJobConfigsV2,
)

# This job_type contains the default settings for compression and Code Ocean
# pipelines. We recommend creating a custom one for your experiments. Reach
# out to Scientific Computing for more information.
job_type = "default"

# acq_datetime = datetime(2025, 4, 25, 16, 41, 23)
acq_datetime = datetime(2020, 4, 25, 16, 41, 0)

# Compression settings. As job_type default, no compression will be performed.
pophys_task = Task(
    job_settings={
        "input_source": (
            "/allen/aind/scratch/svc_aind_upload/test_data_sets/bci/"
            "042525/042525/pophys"
        )
    }
)

modality_transformation_settings = {"pophys": pophys_task}

# 1. Define the JobSettings for the GatherMetadataJob in aind-metadata-mapper
bergamo_session_settings = BergamoSessionSettings(
    input_source=(
        "/allen/aind/scratch/svc_aind_upload/test_data_sets/bci/"
        "042525/042525/pophys"
    ),
    experimenter_full_name=["John Apple"],
    subject_id="784746",
    imaging_laser_wavelength=920,
    fov_imaging_depth=100,
    fov_targeted_structure="Primary Motor Cortex",
    notes="test upload",
)

# 2. Define SessionSettings object with defined job settings
session_settings = SessionSettings(job_settings=bergamo_session_settings)

# 3. Define GatherMetadataJobSettings with session_settings.
# %OUTPUT_LOCATION is a special string that will be replaced by the service
# with the correct staging location.
metadata_job_settings = GatherMetadataJobSettings(
    directory_to_write_to="%OUTPUT_LOCATION",
    session_settings=session_settings,
    metadata_dir=(
        "/allen/aind/scratch/svc_aind_upload/test_data_sets/bci/042525/042525"
    ),
)

gather_preliminary_metadata = Task(
    job_settings=metadata_job_settings.model_dump(
        mode="json", exclude_none=True
    )
)


upload_job_configs_v2 = UploadJobConfigsV2(
    job_type=job_type,
    project_name="Brain Computer Interface",
    platform=Platform.SINGLE_PLANE_OPHYS,
    modalities=[Modality.POPHYS],
    subject_id="784746",
    acq_datetime=acq_datetime,
    tasks={
        "modality_transformation_settings": modality_transformation_settings,
        "gather_preliminary_metadata": gather_preliminary_metadata,
    },
)

submit_request_v2 = SubmitJobRequestV2(
    upload_jobs=[upload_job_configs_v2],
)

post_request_content = submit_request_v2.model_dump(
    mode="json", exclude_none=True
)

# Please use the production endpoint for submitting jobs and the dev endpoint
# for running tests.
# endpoint = "http://aind-data-transfer-service"
endpoint = "http://aind-data-transfer-service-dev"  # For testing

submit_job_response = requests.post(
    url=f"{endpoint}/api/v2/submit_jobs",
    json=post_request_content,
)
print(submit_job_response.status_code)
print(submit_job_response.json())



"""
This example demonstrates how to set custom Slurm settings to request
additional time for the compression step and additional memory.
"""

from datetime import datetime

import requests
from aind_data_schema_models.modalities import Modality
from aind_data_schema_models.platforms import Platform

from aind_data_transfer_service.models.core import (
    SubmitJobRequestV2,
    Task,
    UploadJobConfigsV2,
)

# The job_type contains the default settings for compression and Code Ocean
# pipelines.
job_type = "ecephys"

acq_datetime = datetime(2023, 4, 3, 18, 17, 7)

# If a job is special and requires more slurm resources, then they can be
# defined here. For extra validation, you can pip install aind-slurm-rest-v2
# and import it as:
# from aind_slurm_rest_v2.models.v0040_job_desc_msg import V0040JobDescMsg

ecephys_task = Task(
    image_resources={
        "memory_per_cpu": {"set": True, "number": 6000},  # In MB
        "minimum_cpus_per_node": 6,
        "time_limit": {"set": True, "number": 240},  # In minutes
    },
    job_settings={
        "input_source": (
            "/allen/aind/scratch/svc_aind_upload/test_data_sets/"
            "ecephys/655019_2023-04-03_18-17-07"
        )
    },
)

modality_transformation_settings = {"ecephys": ecephys_task}

gather_preliminary_metadata = Task(
    job_settings={
        "metadata_dir": (
            "/allen/aind/scratch/svc_aind_upload/test_data_sets/"
            "ecephys/655019_2023-04-03_18-17-07"
        )
    }
)


upload_job_configs_v2 = UploadJobConfigsV2(
    job_type=job_type,
    project_name="Ephys Platform",
    platform=Platform.ECEPHYS,
    modalities=[Modality.ECEPHYS],
    subject_id="655019",
    acq_datetime=acq_datetime,
    tasks={
        "modality_transformation_settings": modality_transformation_settings,
        "gather_preliminary_metadata": gather_preliminary_metadata,
    },
)

submit_request_v2 = SubmitJobRequestV2(
    upload_jobs=[upload_job_configs_v2],
)

post_request_content = submit_request_v2.model_dump(
    mode="json", exclude_none=True
)

# Please use the production endpoint for submitting jobs and the dev endpoint
# for running tests.
# endpoint = "http://aind-data-transfer-service"
endpoint = "http://aind-data-transfer-service-dev"  # For testing

submit_job_response = requests.post(
    url=f"{endpoint}/api/v2/submit_jobs",
    json=post_request_content,
)
print(submit_job_response.status_code)
print(submit_job_response.json())



"""This example demonstrates how to submit an HCR job."""

from datetime import datetime

import requests
from aind_data_schema_models.modalities import Modality
from aind_data_schema_models.platforms import Platform

from aind_data_transfer_service.models.core import (
    SubmitJobRequestV2,
    Task,
    UploadJobConfigsV2,
)

job_type = "HCR"

acq_datetime = datetime(2025, 3, 10, 12, 0, 9)

num_of_partitions: int = 4

# Add custom job_settings for the hcr image python command.
spim_task = Task(
    image_resources={"array": f"0-{num_of_partitions - 1}"},
    job_settings={
        "input_source": (
            "/allen/aind/scratch/svc_aind_upload/test_data_sets/"
            "HCR_772646_2025-03-10_12-00-00"
        ),
        "num_of_partitions": num_of_partitions,
    },
)

modality_settings = {
    Modality.SPIM.abbreviation: spim_task,
}

gather_preliminary_metadata = Task(
    job_settings={
        "metadata_dir": (
            "/allen/aind/scratch/svc_aind_upload/test_data_sets/"
            "HCR_772646_2025-03-10_12-00-00"
        )
    }
)

# The job_type loads defaults settings from AWS Parameter Store
upload_job_configs_v2 = UploadJobConfigsV2(
    job_type=job_type,
    s3_bucket="open",
    project_name="MSMA Platform",
    platform=Platform.HCR,
    modalities=[Modality.SPIM],
    subject_id="772646",
    acq_datetime=acq_datetime,
    tasks={
        "modality_transformation_settings": modality_settings,
        "gather_preliminary_metadata": gather_preliminary_metadata,
    },
)

upload_jobs = [upload_job_configs_v2]

submit_request = SubmitJobRequestV2(
    upload_jobs=upload_jobs,
)

post_request_content = submit_request.model_dump(
    mode="json", exclude_none=True
)

# Please use the production endpoint for submitting jobs and the dev endpoint
# for running tests.
# endpoint = "http://aind-data-transfer-service"
endpoint = "http://aind-data-transfer-service-dev"  # For testing

submit_job_response = requests.post(
    url=f"{endpoint}/api/v2/submit_jobs",
    json=post_request_content,
)
print(submit_job_response.status_code)
print(submit_job_response.json())
